/******************************************************************************
SHOW.C
      Image Acquisition & Display Program - Primitive Control Functions
                       Runs on any VGA or MCGA display.
 Displays the image with the serial register on the right edge of the screen.
 This program displays an image stored in a data file whose name is given on
 the command line. This program does not operate the cameras. Store.c stores
 a camera image in a data file, which can be used to create the data file for
 this program. 
 NOTE- program assumes a data file of 320X200 pixels.
 						Kev Forrester Jan 14 92

******************************************************************************/
#include <bios.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

char *fname;

char far *screen_row[200] = { 0, };   /* Pointers to graphics display rows */
/* screen_row[] is initialized in main() to point to the start of each row on
   the VGA display.  It is then used by the show_image() function to update
   the VGA memory. */

unsigned huge buffer[65536] = { 0, }, /* Camera Image Buffer */
         far *ptr[320] = { 0, },      /* Image Display Pointers */
         far *ptable[32] = { 0,};     /* Image Collection Pointers */
/* buffer[] is an array of 64 K pixels (128 K bytes) that is used to collect
   data from the camera.  The ptable[] array is used by the AT200 control
   library functions to collect the pixels.  It is initialized in main() to
   point to each 2 K pixel (4 K byte) block in the buffer[] array.  The ptr[]
   array is used by show_image() to display 320 pixel rows on the VGA.  It is
   also initialized in main(). */

void mcga_display_mode(int mode)
{ /* Select text mode or 320 x 200 - 256 color graphics mode */
union REGS regs;

  regs.x.ax = (mode) ? 19 : 3;  /* AH=0, AL=Mode */
  int86(16, &regs, &regs);      /* Call video bios */
}

void mcga_color_set(int color,int red,int green,int blue)
{ /* Set the RGB values for a color in 256 color graphics modes */
  outp(0x3C8,color);  /* DAC color control register selects the color */
  outp(0x3C9,red);    /* Set Red DAC (0..63) */
  outp(0x3C9,green);  /* Set Green DAC (0..63) */
  outp(0x3C9,blue);   /* Set Blue DAC (0..63) */
}

void far *normalize_pointer(void far *data)
/* Make pointers have the lowest offset possible (and the largest segment).
   This function is needed to insure that each pointer in the ptable[] array
   can be used for 2 K pixels (4 K bytes).  Taking the address of an element
   of a huge array only guarantees that the segment:offset pair will be valid
   for that element.  The calculation performed here guarantees that the
   segment:offset pair will be valid for at least the 4 K addresses we need.
   See the Microsoft Macro Assembler 5.1 manual, section 13.2 for more
   information on REAL MODE ADDRESSING. */
{
unsigned seg,offs;

  seg  = (unsigned) ((long) data >> 16L);
  offs = (unsigned) ((long) data & 0xFFFF);
  seg += offs >> 4;
  offs &= 15;
  return( (void far *) (((long) seg << 16L) | (long) offs) );
}

void show_image(void)
{
unsigned char buff[320];
unsigned max,min;
register int x,y;
unsigned count;
int fd;
/* write buffer to file picture */
	fd = open(fname, O_RDONLY | O_BINARY , S_IREAD | S_IWRITE);
	for(x=0;x<32;x++)
	count=read(fd,ptable[x],4096);
	close(fd);

  for( min=65535,max=y=0 ; y<200 ; y++ )
    for( x=319 ; x>=0 ; x-- ) {
      if( max < ptr[x][y] )
        max = ptr[x][y];
      if( min > ptr[x][y] )
        min = ptr[x][y];
    }
  max = (unsigned) (((long) max - (long) min + 62L) / 63L);
  if( !max )
    max = 1;    /* Don't divide by zero */
  for( y=0 ; y<200 ; y++ ) {
    for( x=319 ; x>=0 ; x-- )
      buff[319-x] = (unsigned char) ((ptr[x][y] - min) / max);
    memcpy(screen_row[y],buff,320);
  }
}

void main(int argc,char *argv[])
{
register unsigned x;

  if(argc!=2) {
printf("\nYou must type show and then the name of the"
      " file you wish to display!.\n");
  	exit(0);
	}

  fname=argv[1];
  for( x=0 ; x<32 ; x++ )     /* Load image acquisition array */
    ptable[x] = normalize_pointer(&buffer[x * 2048]);
  for( x=0 ; x<320 ; x++ )    /* Load image display array */
    ptr[x] = normalize_pointer(&buffer[x * 200]);
  for( x=0 ; x<200 ; x++ )    /* Load graphics display array */
    screen_row[x] = &((char far *)0xA0000000)[x * 320];
  mcga_display_mode( 1 );
  for( x=0 ; x<64 ; x++ )
    mcga_color_set(x,x,x,x);
    show_image();
      x = getch();
  mcga_display_mode( 0 );
  exit( 0 );
}
